// Vikunja is a to-do list application to facilitate your life.
// Copyright 2018-present Vikunja and contributors. All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public Licensee as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public Licensee for more details.
//
// You should have received a copy of the GNU Affero General Public Licensee
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package events

import "github.com/ThreeDotsLabs/watermill/message"

// Listener represents something that listens to events
type Listener interface {
	Handle(msg *message.Message) error
	Name() string
}

var listeners map[string][]Listener

func init() {
	listeners = make(map[string][]Listener)
}

// RegisterListener is used to register a listener when a specific event happens
func RegisterListener(name string, listener Listener) {
	listeners[name] = append(listeners[name], listener)
}
